import { createTRPCHandle } from "trpc-sveltekit";
import { initTRPC } from "@trpc/server";
import { z } from "zod";
import { Connection, PublicKey, LAMPORTS_PER_SOL, VOTE_PROGRAM_ID } from "@solana/web3.js";
import { p as protonParsers } from "./types.js";
import "@onsol/tldparser";
import { getAssetAccountDataSerializer } from "@nifty-oss/asset";
import { g as getRPCUrl } from "./get-rpc-url.js";
import { H as HELIUS_API_KEY, B as BIRDEYE_API_KEY } from "./private.js";
import { ConcurrentMerkleTreeAccount } from "@solana/spl-account-compression";
import { Helius } from "helius-sdk";
const parseTransaction = (transaction2, address) => {
  let parser = protonParsers.UNKNOWN;
  const transactionType = transaction2.type;
  if (typeof protonParsers[transactionType] === "undefined") {
    return protonParsers.UNKNOWN(transaction2, address);
  }
  parser = protonParsers[transactionType];
  try {
    return parser(transaction2, address);
  } catch (error) {
    console.log(error);
    return protonParsers.UNKNOWN(transaction2, address);
  }
};
async function createContext(event) {
  return {
    // context information
  };
}
const t$1 = initTRPC.context().create();
const accountInfo = t$1.procedure.input(z.tuple([z.string(), z.boolean()])).query(async ({ input }) => {
  const [address, isMainnet] = input;
  const connection = new Connection(
    getRPCUrl(`?api-key=${HELIUS_API_KEY}`, isMainnet),
    "confirmed"
  );
  const pubKey = new PublicKey(address);
  const accountInfo2 = await connection.getParsedAccountInfo(pubKey);
  return {
    ...accountInfo2,
    balance: (accountInfo2?.value?.lamports || 0) / LAMPORTS_PER_SOL
  };
});
const getSolanaDomain = async (usernames, address = "") => {
  const url = `https://api.helius.xyz/v0/addresses/${address}/names?api-key=${HELIUS_API_KEY}`;
  const response = await fetch(url);
  const data = await response.json();
  if (data?.domainNames) {
    for (const domain of data.domainNames) {
      usernames.push({
        type: "bonfida",
        username: `${domain}.sol`
      });
    }
  }
};
const accountUsernames = t$1.procedure.input(z.string()).output(
  z.array(
    z.object({
      type: z.string(),
      username: z.string()
    })
  )
).query(async ({ input: address }) => {
  const usernames = [];
  await getSolanaDomain(usernames, address);
  return usernames || [];
});
const asset = t$1.procedure.input(z.tuple([z.string(), z.boolean()])).query(async ({ input }) => {
  const [asset2, isMainnet] = input;
  const url = getRPCUrl(`?api-key=${HELIUS_API_KEY}`, isMainnet);
  const response = await fetch(url, {
    body: JSON.stringify({
      id: "asset",
      jsonrpc: "2.0",
      method: "getAsset",
      params: {
        id: asset2,
        options: {
          showFungible: true
        }
      }
    }),
    headers: {
      "Content-Type": "application/json"
    },
    method: "POST"
  });
  const data = await response.json();
  let metadata;
  if (data?.result?.compression?.compressed === true) {
    const assetData = await fetch(data.result.content.json_uri);
    const returnAssetData = await assetData.json();
    metadata = {
      address: data?.result?.id || "",
      assetHash: data?.result?.compression?.asset_hash,
      attributes: returnAssetData?.attributes || [],
      burnt: data?.result?.burnt,
      collectionKey: data?.result?.grouping[0]?.group_value || "",
      compressed: true,
      creatorHash: data?.result?.compression?.creator_hash,
      creators: data?.result?.creators || [],
      dataHash: data?.result?.compression?.data_hash,
      delegate: data?.result?.ownership?.delegated ? data?.result?.ownership?.delegate : "",
      description: returnAssetData?.description || "",
      frozen: data?.result?.ownership?.frozen,
      image: returnAssetData?.image || "",
      leafId: data?.result?.compression?.leaf_id,
      mintExtensions: data?.result?.mint_extensions || "",
      mutable: data?.result?.mutable,
      name: returnAssetData?.name || "",
      owner: data?.result?.ownership?.owner || "",
      // sellerFeeBasisPoints ??
      sellerFeeBasisPoints: data?.result?.royalty.basis_points || 0,
      seq: data?.result?.compression?.seq,
      tree: data?.result?.compression?.tree
    };
  }
  return metadata ?? data?.result;
});
function getAPIUrl(path, isMainnet) {
  const baseUrl = isMainnet ? "https://api.helius.xyz" : "https://api-devnet.helius.xyz";
  return `${baseUrl}${path}`;
}
const balances = t$1.procedure.input(z.tuple([z.string(), z.boolean()])).query(async ({ input }) => {
  const [account, isMainnet] = input;
  const url = getAPIUrl(
    `/v0/addresses/${account}/balances?api-key=${HELIUS_API_KEY}`,
    isMainnet
  );
  const response = await fetch(url);
  const data = await response.json();
  return data;
});
const concurrentMerkleTree = t$1.procedure.input(z.object({ address: z.string(), isMainnet: z.boolean() })).query(async ({ input }) => {
  const connection = new Connection(
    getRPCUrl(`?api-key=${HELIUS_API_KEY}`, input.isMainnet),
    "confirmed"
  );
  const pubkey = new PublicKey(input.address);
  const cmt = await ConcurrentMerkleTreeAccount.fromAccountAddress(
    connection,
    pubkey
  );
  const authority = cmt.getAuthority();
  const root = cmt.getCurrentRoot();
  const seq = cmt.getCurrentSeq().toString();
  const canopyDepth = cmt.getCanopyDepth();
  const maxBufferSize = cmt.getMaxBufferSize();
  const treeHeight = cmt.getMaxDepth();
  const creationSlot = cmt.getCreationSlot().toNumber();
  const rightMostIndex = cmt.tree.rightMostPath.index;
  return {
    authority,
    canopyDepth,
    creationSlot,
    maxBufferSize,
    rightMostIndex,
    root,
    seq,
    treeHeight
  };
});
const currentSlot = t$1.procedure.input(z.tuple([z.boolean()])).query(async ({ input }) => {
  const [isMainnet] = input;
  const connection = new Connection(
    getRPCUrl(`?api-key=${HELIUS_API_KEY}`, isMainnet),
    "confirmed"
  );
  const slot = await connection.getSlot();
  return slot;
});
const deprecatedImage = t$1.procedure.input(z.string()).query(async ({ input: account }) => {
  const response = await fetch(
    `https://github.com/solana-labs/token-list/blob/main/assets/mainnet/${account}/logo.png?raw=true`
  );
  const contentType = response.headers.get("content-type");
  if (contentType?.includes("image/png")) {
    return `https://github.com/solana-labs/token-list/blob/main/assets/mainnet/${account}/logo.png?raw=true`;
  } else {
    const url = getAPIUrl(
      `/v0/token-metadata/?api-key=${HELIUS_API_KEY}`,
      true
    );
    const response2 = await fetch(url, {
      body: JSON.stringify({
        includeOffChain: true,
        mintAccounts: [account]
      }),
      method: "POST"
    });
    const json = await response2.json();
    return json[0]?.legacyMetadata?.logoURI ?? "";
  }
});
const niftyAsset = t$1.procedure.input(z.tuple([z.string(), z.boolean()])).query(async ({ input }) => {
  const [address, isMainnet] = input;
  const connection = new Connection(
    getRPCUrl(`?api-key=${HELIUS_API_KEY}`, isMainnet),
    "confirmed"
  );
  const pubKey = new PublicKey(address);
  const accountInfo2 = await connection.getAccountInfo(pubKey);
  if (accountInfo2) {
    return getAssetAccountDataSerializer().deserialize(
      accountInfo2.data
    );
  }
  return null;
});
const price = t$1.procedure.input(z.string()).query(async ({ input: token2 }) => {
  const response = await fetch(
    `https://public-api.birdeye.so/defi/price/?address=${token2}`,
    {
      headers: {
        Accept: "application/json",
        "X-API-KEY": BIRDEYE_API_KEY,
        "x-chain": "solana"
      }
    }
  );
  const json = await response.json();
  return json?.data?.value || 0;
});
const rawTransaction = t$1.procedure.input(z.tuple([z.string(), z.boolean()])).query(async ({ input }) => {
  try {
    const [signature, isMainnet] = input;
    const connection = new Connection(
      getRPCUrl(`?api-key=${HELIUS_API_KEY}`, isMainnet),
      "confirmed"
    );
    const transaction2 = await connection.getTransaction(signature, {
      maxSupportedTransactionVersion: 0
    });
    if (!transaction2) {
      return { data: null, error: "Raw transaction not found" };
    }
    return {
      transaction: transaction2
    };
  } catch (error) {
    return { data: null, error: "Server error" };
  }
});
const searchAssets = t$1.procedure.input(
  z.object({
    account: z.string(),
    cursor: z.number().optional(),
    isMainnet: z.boolean(),
    nativeBalance: z.boolean().optional(),
    tokenType: z.string().optional()
  })
).query(async ({ input }) => {
  const {
    account,
    cursor = 1,
    isMainnet,
    tokenType = "all",
    nativeBalance = false
  } = input;
  const url = getRPCUrl(`/?api-key=${HELIUS_API_KEY}`, isMainnet);
  const response = await fetch(url, {
    body: JSON.stringify({
      id: "my-id",
      jsonrpc: "2.0",
      method: "searchAssets",
      params: {
        displayOptions: {
          showNativeBalance: nativeBalance
        },
        limit: 1e3,
        ownerAddress: account,
        page: cursor,
        sortBy: { sortBy: "id", sortDirection: "asc" },
        tokenType
      }
    }),
    headers: {
      "Content-Type": "application/json"
    },
    method: "POST"
  });
  const { result } = await response.json();
  return result;
});
const token = t$1.procedure.input(z.tuple([z.string(), z.boolean()])).query(async ({ input }) => {
  const [token2, isMainnet] = input;
  const url = getAPIUrl(
    `/v0/token-metadata/?api-key=${HELIUS_API_KEY}`,
    isMainnet
  );
  const response = await fetch(url, {
    body: JSON.stringify({
      includeOffChain: true,
      mintAccounts: [token2]
    }),
    method: "POST"
  });
  const json = await response.json();
  return json;
});
const tps = t$1.procedure.input(z.boolean()).query(async ({ input }) => {
  let helius;
  if (input == true) {
    helius = new Helius(HELIUS_API_KEY);
  } else {
    helius = new Helius(HELIUS_API_KEY, "devnet");
  }
  const tps2 = await helius.rpc.getCurrentTPS();
  return tps2;
});
const transaction = t$1.procedure.input(
  z.object({
    account: z.string().optional(),
    isMainnet: z.boolean(),
    transaction: z.string()
  })
).query(async ({ input }) => {
  try {
    const url = getAPIUrl(
      `/v0/transactions/?api-key=${HELIUS_API_KEY}`,
      input.isMainnet
    );
    const response = await fetch(url, {
      body: JSON.stringify({
        transactions: [input?.transaction]
      }),
      method: "POST"
    });
    if (!response.ok) {
      return { data: null, error: "Transaction not found" };
    }
    const [tx] = await response.json();
    const parsed = parseTransaction(tx, input?.account);
    if (parsed === void 0) {
      return { data: null, error: "Transaction not found" };
    }
    parsed.raw = tx;
    return parsed;
  } catch (error) {
    return { data: null, error: "Server error" };
  }
});
const transactions = t$1.procedure.input(
  z.object({
    account: z.string(),
    cursor: z.string().optional(),
    filter: z.string().optional(),
    isMainnet: z.boolean(),
    user: z.string().optional()
  })
).output(
  z.object({
    oldest: z.string(),
    result: z.array(
      z.object({
        accounts: z.array(
          z.object({
            account: z.string(),
            changes: z.array(
              z.object({
                amount: z.number(),
                mint: z.string()
              })
            )
          })
        ),
        actions: z.array(
          z.object({
            actionType: z.string(),
            amount: z.number(),
            from: z.string().nullable().optional(),
            fromName: z.string().optional(),
            received: z.string().optional(),
            sent: z.string().optional(),
            to: z.string(),
            toName: z.string().optional()
          })
        ),
        fee: z.number(),
        primaryUser: z.string(),
        raw: z.any(),
        signature: z.string(),
        source: z.string(),
        timestamp: z.number(),
        type: z.string()
      })
    )
  })
).query(async ({ input }) => {
  const url = getAPIUrl(
    `/v0/addresses/${input.account}/transactions?api-key=${HELIUS_API_KEY}${input.filter ? `&type=${input.filter}` : ""}${input.cursor ? `&before=${input.cursor}` : ""}`,
    input.isMainnet
  );
  const response = await fetch(url);
  const json = await response.json();
  const result = json.map((tx) => parseTransaction(tx, input.user)) || [];
  return {
    oldest: json[json.length - 1]?.signature || "",
    result
  };
});
const voteFilter = VOTE_PROGRAM_ID.toBase58();
const blockTransactions = t$1.procedure.input(
  z.object({
    cursor: z.string().optional(),
    isMainnet: z.boolean(),
    limit: z.number().min(1).max(100).optional(),
    slot: z.number()
  })
).output(
  z.object({
    oldest: z.string(),
    result: z.array(
      z.object({
        accounts: z.array(
          z.object({
            account: z.string(),
            changes: z.array(
              z.object({
                amount: z.number(),
                mint: z.string()
              })
            )
          })
        ),
        actions: z.array(
          z.object({
            actionType: z.string(),
            amount: z.number(),
            from: z.string(),
            fromName: z.string().optional(),
            received: z.string().optional(),
            sent: z.string().optional(),
            to: z.string(),
            toName: z.string().optional()
          })
        ),
        fee: z.number(),
        primaryUser: z.string(),
        raw: z.any(),
        signature: z.string(),
        source: z.string(),
        timestamp: z.number(),
        type: z.string()
      })
    )
  })
).query(async ({ input }) => {
  const limit = input.limit ?? 100;
  const invokedPrograms = /* @__PURE__ */ new Map();
  const connection = new Connection(
    getRPCUrl(`?api-key=${HELIUS_API_KEY}`, input.isMainnet),
    "confirmed"
  );
  const block = await connection.getBlock(input.slot, {
    maxSupportedTransactionVersion: 0
  });
  const transactions2 = block?.transactions.map((tx, index) => {
    let signature;
    if (tx.transaction.signatures.length > 0) {
      signature = tx.transaction.signatures[0];
    }
    const programIndexes = tx.transaction.message.compiledInstructions.map((ix) => ix.programIdIndex).concat(
      tx.meta?.innerInstructions?.flatMap((ix) => {
        return ix.instructions.map(
          (ix2) => ix2.programIdIndex
        );
      }) || []
    );
    const invocations = programIndexes.reduce(
      (acc, programIndex) => {
        const programId = tx.transaction.message.getAccountKeys({
          accountKeysFromLookups: tx.meta?.loadedAddresses
        }).get(programIndex).toBase58();
        const programTransactionCount = invokedPrograms.get(programId) || 0;
        invokedPrograms.set(
          programId,
          programTransactionCount + 1
        );
        const count = acc.get(programId) || 0;
        acc.set(programId, count + 1);
        return acc;
      },
      /* @__PURE__ */ new Map()
    );
    return {
      index,
      invocations,
      meta: tx.meta,
      signature
    };
  });
  let signatureList = transactions2?.filter(
    ({ invocations }) => !(invocations.has(voteFilter) && invocations.size === 1)
  ).map(({ signature }) => signature);
  if (!signatureList?.length) {
    return {
      oldest: "",
      result: []
    };
  }
  if (input.cursor) {
    const lastTransactionIndex = signatureList.indexOf(input.cursor);
    if (lastTransactionIndex >= 0) {
      signatureList = signatureList.slice(lastTransactionIndex + 1);
    }
  }
  signatureList = signatureList.slice(0, limit);
  const url = `https://api.helius.xyz/v0/transactions/?api-key=${HELIUS_API_KEY}`;
  const response = await fetch(url, {
    body: JSON.stringify({
      transactions: signatureList
    }),
    method: "POST"
  });
  const json = await response.json();
  const result = json.map((tx) => parseTransaction(tx)) || [];
  return {
    oldest: signatureList?.slice(-1)?.[0] || "",
    result
  };
});
const assets = t$1.procedure.input(
  z.object({
    account: z.string(),
    cursor: z.number().optional(),
    isMainnet: z.boolean()
  })
).query(async ({ input }) => {
  const { cursor = 1, account } = input;
  const url = getRPCUrl(`?api-key=${HELIUS_API_KEY}`, input.isMainnet);
  const response = await fetch(url, {
    body: JSON.stringify({
      id: "get-assets-" + account,
      jsonrpc: "2.0",
      method: "getAssetsByOwner",
      params: {
        limit: 1e3,
        ownerAddress: account,
        page: cursor,
        sortBy: {
          sortBy: "created",
          sortDirection: "desc"
        }
      }
    }),
    headers: {
      "Content-Type": "application/json"
    },
    method: "POST"
  });
  const result = await response.json();
  return result?.result;
});
const cnftTransactions = t$1.procedure.input(
  z.object({
    account: z.string(),
    cursor: z.number().optional().default(1),
    isMainnet: z.boolean()
  })
).query(async ({ input }) => {
  const url = getRPCUrl(`?api-key=${HELIUS_API_KEY}`, input.isMainnet);
  const response = await fetch(url, {
    body: JSON.stringify({
      id: "signatures",
      jsonrpc: "2.0",
      method: "getSignaturesForAsset",
      params: {
        id: input.account,
        limit: 50,
        page: input.cursor
      }
    }),
    headers: {
      "Content-Type": "application/json"
    },
    method: "POST"
  }).then((res) => res.json());
  const signatures = response.result.items.map(
    ([signature]) => signature
  );
  const transactionUrl = getAPIUrl(
    `/v0/transactions/?api-key=${HELIUS_API_KEY}`,
    input.isMainnet
  );
  const transactions2 = await fetch(transactionUrl, {
    body: JSON.stringify({
      transactions: signatures
    }),
    method: "POST"
  }).then((res) => res.json()) || [];
  const result = transactions2.map((tx) => parseTransaction(tx)) || [];
  return {
    oldest: signatures[signatures.length - 1],
    result
  };
});
const t = initTRPC.context().create();
const router = t.router({
  accountInfo,
  accountUsernames,
  asset,
  assets,
  balances,
  blockTransactions,
  cnftTransactions,
  concurrentMerkleTree,
  currentSlot,
  deprecatedImage,
  niftyAsset,
  price,
  rawTransaction,
  searchAssets,
  token,
  tps,
  transaction,
  transactions
});
const ONE_DAY_IN_SECONDS = 60 * 60 * 24;
const handle = createTRPCHandle({
  router,
  createContext,
  responseMeta({ type, errors }) {
    if (type === "query" && errors.length === 0) {
      return {
        headers: {
          "cache-control": `s-maxage=1, stale-while-revalidate=${ONE_DAY_IN_SECONDS}`
        }
      };
    }
    return {};
  }
});
export {
  handle
};
