import { g as get_store_value, c as create_ssr_component, b as subscribe, f as createEventDispatcher, a as add_attribute, d as each, e as escape, v as validate_component } from "./index3.js";
import { r as recentSearchesKey } from "./config.js";
import { I as Icon } from "./icon.js";
import { s as shortenString } from "./shorten-string.js";
import { WalletError, WalletNotConnectedError, WalletReadyState, WalletNotReadyError } from "@solana/wallet-adapter-base";
import { w as writable } from "./index2.js";
class WalletNotSelectedError extends WalletError {
  constructor() {
    super(...arguments);
    this.name = "WalletNotSelectedError";
  }
}
function setLocalStorage(key, value = null) {
  try {
    if (value === null) {
      localStorage.removeItem(key);
    } else {
      localStorage.setItem(key, JSON.stringify(value));
    }
  } catch (error) {
    if (typeof window !== "undefined") {
      console.error(error);
    }
  }
}
const walletStore = createWalletStore();
function addAdapterEventListeners(adapter) {
  const { onError, wallets } = get_store_value(walletStore);
  wallets.forEach(({ adapter: adapter2 }) => {
    adapter2.on("readyStateChange", onReadyStateChange, adapter2);
  });
  adapter.on("connect", onConnect);
  adapter.on("disconnect", onDisconnect);
  adapter.on("error", onError);
}
async function autoConnect() {
  const { adapter } = get_store_value(walletStore);
  try {
    walletStore.setConnecting(true);
    await (adapter === null || adapter === void 0 ? void 0 : adapter.connect());
  } catch (error) {
    walletStore.resetWallet();
  } finally {
    walletStore.setConnecting(false);
  }
}
async function connect() {
  const { connected, connecting, disconnecting, ready, adapter } = get_store_value(walletStore);
  if (connected || connecting || disconnecting)
    return;
  if (!adapter)
    throw newError(new WalletNotSelectedError());
  if (!(ready === WalletReadyState.Installed || ready === WalletReadyState.Loadable)) {
    walletStore.resetWallet();
    if (typeof window !== "undefined") {
      window.open(adapter.url, "_blank");
    }
    throw newError(new WalletNotReadyError());
  }
  try {
    walletStore.setConnecting(true);
    await adapter.connect();
  } catch (error) {
    walletStore.resetWallet();
    throw error;
  } finally {
    walletStore.setConnecting(false);
  }
}
function createWalletStore() {
  const { subscribe: subscribe2, update } = writable({
    autoConnect: false,
    wallets: [],
    adapter: null,
    connected: false,
    connecting: false,
    disconnecting: false,
    localStorageKey: "walletAdapter",
    onError: (error) => console.error(error),
    publicKey: null,
    ready: "Unsupported",
    wallet: null,
    name: null,
    walletsByName: {},
    connect,
    disconnect,
    select,
    sendTransaction,
    signTransaction: void 0,
    signAllTransactions: void 0,
    signMessage: void 0
  });
  function updateWalletState(adapter) {
    updateAdapter(adapter);
    update((store) => ({
      ...store,
      name: (adapter === null || adapter === void 0 ? void 0 : adapter.name) || null,
      wallet: adapter,
      ready: (adapter === null || adapter === void 0 ? void 0 : adapter.readyState) || "Unsupported",
      publicKey: (adapter === null || adapter === void 0 ? void 0 : adapter.publicKey) || null,
      connected: (adapter === null || adapter === void 0 ? void 0 : adapter.connected) || false
    }));
    if (!adapter)
      return;
    if (shouldAutoConnect()) {
      autoConnect();
    }
  }
  function updateWalletName(name) {
    var _a;
    const { localStorageKey, walletsByName } = get_store_value(walletStore);
    const adapter = (_a = walletsByName === null || walletsByName === void 0 ? void 0 : walletsByName[name]) !== null && _a !== void 0 ? _a : null;
    setLocalStorage(localStorageKey, name);
    updateWalletState(adapter);
  }
  function updateAdapter(adapter) {
    removeAdapterEventListeners();
    let signTransaction = void 0;
    let signAllTransactions = void 0;
    let signMessage = void 0;
    if (adapter) {
      if ("signTransaction" in adapter) {
        signTransaction = async function(transaction) {
          const { connected } = get_store_value(walletStore);
          if (!connected)
            throw newError(new WalletNotConnectedError());
          return await adapter.signTransaction(transaction);
        };
      }
      if ("signAllTransactions" in adapter) {
        signAllTransactions = async function(transactions) {
          const { connected } = get_store_value(walletStore);
          if (!connected)
            throw newError(new WalletNotConnectedError());
          return await adapter.signAllTransactions(transactions);
        };
      }
      if ("signMessage" in adapter) {
        signMessage = async function(message) {
          const { connected } = get_store_value(walletStore);
          if (!connected)
            throw newError(new WalletNotConnectedError());
          return await adapter.signMessage(message);
        };
      }
      addAdapterEventListeners(adapter);
    }
    update((store) => ({ ...store, adapter, signTransaction, signAllTransactions, signMessage }));
  }
  return {
    resetWallet: () => updateWalletName(null),
    setConnecting: (connecting) => update((store) => ({ ...store, connecting })),
    setDisconnecting: (disconnecting) => update((store) => ({ ...store, disconnecting })),
    setReady: (ready) => update((store) => ({ ...store, ready })),
    subscribe: subscribe2,
    updateConfig: (walletConfig) => update((store) => ({
      ...store,
      ...walletConfig
    })),
    updateWallets: (wallets) => update((store) => ({ ...store, ...wallets })),
    updateStatus: (walletStatus) => update((store) => ({ ...store, ...walletStatus })),
    updateWallet: (walletName) => updateWalletName(walletName)
  };
}
async function disconnect() {
  const { disconnecting, adapter } = get_store_value(walletStore);
  if (disconnecting)
    return;
  if (!adapter)
    return walletStore.resetWallet();
  try {
    walletStore.setDisconnecting(true);
    await adapter.disconnect();
  } finally {
    walletStore.resetWallet();
    walletStore.setDisconnecting(false);
  }
}
function newError(error) {
  const { onError } = get_store_value(walletStore);
  onError(error);
  return error;
}
function onConnect() {
  const { adapter } = get_store_value(walletStore);
  if (!adapter)
    return;
  walletStore.updateStatus({
    publicKey: adapter.publicKey,
    connected: adapter.connected
  });
}
function onDisconnect() {
  walletStore.resetWallet();
}
function onReadyStateChange(readyState) {
  const { adapter, wallets } = get_store_value(walletStore);
  if (!adapter)
    return;
  walletStore.setReady(adapter.readyState);
  const walletIndex = wallets.findIndex(({ adapter: adapter2 }) => adapter2.name === this.name);
  if (walletIndex === -1) {
    return;
  } else {
    walletStore.updateWallets([
      ...wallets.slice(0, walletIndex),
      { ...wallets[walletIndex], readyState },
      ...wallets.slice(walletIndex + 1)
    ]);
  }
}
function removeAdapterEventListeners() {
  const { adapter, onError, wallets } = get_store_value(walletStore);
  if (!adapter)
    return;
  wallets.forEach(({ adapter: adapter2 }) => {
    adapter2.off("readyStateChange", onReadyStateChange, adapter2);
  });
  adapter.off("connect", onConnect);
  adapter.off("disconnect", onDisconnect);
  adapter.off("error", onError);
}
async function select(walletName) {
  const { name, adapter } = get_store_value(walletStore);
  if (name === walletName)
    return;
  if (adapter)
    await disconnect();
  walletStore.updateWallet(walletName);
}
async function sendTransaction(transaction, connection, options) {
  const { connected, adapter } = get_store_value(walletStore);
  if (!connected)
    throw newError(new WalletNotConnectedError());
  if (!adapter)
    throw newError(new WalletNotSelectedError());
  return await adapter.sendTransaction(transaction, connection, options);
}
function shouldAutoConnect() {
  const { adapter, autoConnect: autoConnect2, ready, connected, connecting } = get_store_value(walletStore);
  return !(!autoConnect2 || !adapter || !(ready === WalletReadyState.Installed || ready === WalletReadyState.Loadable) || connected || connecting);
}
if (typeof window !== "undefined") {
  window.addEventListener("beforeunload", removeAdapterEventListeners);
}
const search_svelte_svelte_type_style_lang = "";
const css = {
  code: ".input.svelte-19c1mzp{background:rgba(0, 0, 0, 0.95)}",
  map: null
};
const Search = create_ssr_component(($$result, $$props, $$bindings, slots) => {
  let $walletStore, $$unsubscribe_walletStore;
  $$unsubscribe_walletStore = subscribe(walletStore, (value) => $walletStore = value);
  let { inputEl = null } = $$props;
  let { searchError = "" } = $$props;
  let { size = "sm" } = $$props;
  const focusInput = () => {
    inputEl?.focus();
  };
  const clearSearch = () => {
    inputValue = "";
    inputEl?.focus();
  };
  let inputValue = "";
  let connected = false;
  let recent = [];
  createEventDispatcher();
  const getRecentSearches = () => JSON.parse(localStorage.getItem(recentSearchesKey) || "[]");
  const addRecent = (value) => {
    if (!value.search) {
      return;
    }
    const stored = getRecentSearches();
    const exists = stored.find(({ url }) => {
      return url === value.url;
    });
    if (exists) {
      return;
    }
    localStorage.setItem(recentSearchesKey, JSON.stringify([value, ...getRecentSearches()].slice(0, 3)));
  };
  let isMainnetValue = true;
  if ($$props.inputEl === void 0 && $$bindings.inputEl && inputEl !== void 0)
    $$bindings.inputEl(inputEl);
  if ($$props.searchError === void 0 && $$bindings.searchError && searchError !== void 0)
    $$bindings.searchError(searchError);
  if ($$props.size === void 0 && $$bindings.size && size !== void 0)
    $$bindings.size(size);
  if ($$props.focusInput === void 0 && $$bindings.focusInput && focusInput !== void 0)
    $$bindings.focusInput(focusInput);
  if ($$props.clearSearch === void 0 && $$bindings.clearSearch && clearSearch !== void 0)
    $$bindings.clearSearch(clearSearch);
  $$result.css.add(css);
  {
    if ($walletStore.connected && !connected) {
      focusInput();
      const params = new URLSearchParams(window.location.search);
      const network = params.get("network");
      isMainnetValue = network !== "devnet";
      inputValue = $walletStore.publicKey?.toBase58() || "";
      addRecent({
        address: inputValue,
        search: inputValue,
        type: "account",
        url: `/account/${inputValue}?network=${isMainnetValue ? "mainnet" : "devnet"}`,
        valid: true
      });
      window.location.href = `/account/${inputValue}`;
      connected = true;
    }
  }
  $$unsubscribe_walletStore();
  return `<div class="relative z-30 my-2 w-full"><div class="dropdown w-full"><input class="${[
    "input-bordered input h-10 w-full rounded-lg focus:input-primary svelte-19c1mzp",
    size === "lg" ? "h-14" : ""
  ].join(" ").trim()}" placeholder="Input a Solana address, transaction signature, or domain..." tabindex="0" type="text"${add_attribute("this", inputEl, 0)}${add_attribute("value", inputValue, 0)}>
        ${recent.length > 0 && size !== "sm" ? `<ul class="dropdown-content relative my-3 w-full rounded-lg border bg-base-100 p-2 px-4 shadow"><div class="flex flex-wrap items-center justify-between"><p class="text-md mb-1 mt-2">Recents</p>
                    <button class="btn-xs btn border-none bg-transparent"><span class="my-1">Clear all</span></button></div>
                ${recent.length ? `${each(recent, (recentSearch) => {
    return `${recentSearch ? `<li class="m1-ds2 relative z-30 w-full truncate px-0 hover:opacity-60"><a class="block w-full max-w-full text-ellipsis rounded-lg px-1 py-2 text-left hover:bg-secondary" data-sveltekit-reload${add_attribute("href", `${recentSearch.url}?network=${isMainnetValue ? "mainnet" : "devnet"}`, 0)}><div class="flex"><div></div>
                                        <div><p class="text-micro text-xs opacity-50">${escape(shortenString(recentSearch?.address))}</p>
                                            <p class="text-micro text-xs">${escape(recentSearch?.search)}
                                            </p></div>
                                    </div></a>
                            </li>` : ``}`;
  })}` : `<i class="pt-2 text-xs opacity-50">Paste an address or connect a wallet to get started.</i>`}</ul>` : ``}</div>

    <button class="${[
    "btn-ghost btn-sm btn absolute bottom-1/2 right-4 translate-y-1/2 px-2",
    ""
  ].join(" ").trim()}">${`${validate_component(Icon, "Icon").$$render($$result, { id: "search" }, {}, {})}`}</button></div>

${size === "lg" ? `<div class="relative z-10 py-2"><button class="bg-faint btn-outline btn col-span-1 mb-4 w-full"><span class="text-sm">Explore</span></button>
        </div>` : ``}`;
});
export {
  Search as S
};
