import { j as json } from "../../../../../chunks/index.js";
import { PublicKey, Connection } from "@solana/web3.js";
import "../../../../../chunks/types.js";
import { TldParser } from "@onsol/tldparser";
import { ASSET_PROGRAM_ID } from "@nifty-oss/asset";
import { H as HELIUS_API_KEY } from "../../../../../chunks/private.js";
const isValidPublicKey = (address = "") => {
  try {
    return new PublicKey(address.trim());
  } catch (error) {
    return null;
  }
};
const networks = {
  devnet: `https://rpc-devnet.helius.xyz/`,
  mainnet: `https://mainnet.helius-rpc.com`,
  solanaMainnet: "https://api.mainnet-beta.solana.com"
};
const connect = (network = "mainnet", heliusKey) => {
  let url = networks[network];
  if (heliusKey) {
    url += `?api-key=${heliusKey}`;
  }
  return new Connection(url, "confirmed");
};
const recognizedTokens = {
  BONK: "DezXAZ8z7PnrnRJjz3wXBoRgixCa6xjnB7YaB1pPB263",
  FOXY: "FoXyMu5xwXre7zEoSvzViRk3nGawHUp9kUh97y2NDhcq",
  SOL: "So11111111111111111111111111111111111111112",
  USDC: "EPjFWdd5AufqSSqeM2qN1xzybapC8G4wEGGkZwyTDt1v"
};
const getJupiterTokens = async () => {
  try {
    const data = await fetch(`https://token.jup.ag/all`);
    const jsonData = await data.json();
    if (!Array.isArray(jsonData)) {
      throw new Error("Unexpected data format from Jupiter API");
    }
    const tokens = jsonData;
    const tokenMap = tokens.reduce((acc, token) => {
      if (recognizedTokens[token.symbol] && recognizedTokens[token.symbol] === token.address || !recognizedTokens[token.symbol]) {
        acc[token.symbol] = token.address;
      }
      return acc;
    }, {});
    return tokenMap;
  } catch (error) {
    throw new Error(
      `Failed to fetch tokens from Jupiter with error: ${error}`
    );
  }
};
const searchDefaults = {
  address: "",
  search: "",
  type: null,
  url: `/`,
  valid: false
};
const search = async (query, connection) => {
  const ans = new TldParser(connection);
  const probablyTransactionSignature = query.length > 50;
  const probablyBonfidaName = query.length > 4 && query.slice(-4) === ".sol";
  const probablyAnsDomain = query.length > 4 && query.includes(".");
  const tokenSymbols = await getJupiterTokens();
  if (isValidPublicKey(query)) {
    const pubkey = new PublicKey(query);
    const account = await connection.getAccountInfo(pubkey);
    const program = account?.owner.toString();
    let addressType;
    if (account) {
      if (program === "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA" || program === "TokenzQdBNbLqP5VEhdkAS6EPFLC1PHnBqCXEpPxuEb") {
        addressType = "token";
      } else if (program === ASSET_PROGRAM_ID) {
        addressType = "nifty-asset";
      }
    }
    addressType ??= "account";
    return {
      address: query,
      search: query,
      type: addressType,
      url: `/${addressType}/${query}`,
      valid: true
    };
  } else if (probablyTransactionSignature) {
    return {
      address: query,
      search: query,
      type: "transaction",
      url: `/tx/${query}`,
      valid: true
    };
  } else if (probablyBonfidaName) {
    try {
      const resonse = await fetch(
        `https://sns-sdk-proxy.bonfida.workers.dev/resolve/${query?.toLowerCase()}`
      );
      const { result = "" } = await resonse.json();
      return {
        address: result,
        search: query,
        type: "bonfida-domain",
        url: `/account/${result}`,
        valid: true
      };
    } catch (error) {
      return searchDefaults;
    }
  } else if (probablyAnsDomain) {
    const owner = await ans.getOwnerFromDomainTld(query);
    if (!owner) {
      return searchDefaults;
    }
    return {
      address: owner.toBase58(),
      search: query,
      type: "ans-domain",
      url: `/account/${owner}`,
      valid: true
    };
  } else if (Object.keys(tokenSymbols).find((key) => key === query)) {
    return {
      address: tokenSymbols[query],
      search: query,
      type: "token",
      url: `/token/${tokenSymbols[query]}`,
      valid: true
    };
  }
  return searchDefaults;
};
async function GET({ params }) {
  const conection = connect("mainnet", HELIUS_API_KEY);
  const result = await search(params?.query || "", conection);
  return json(result);
}
export {
  GET
};
